/**
 * @file
 * The main code of the zh_rf24 component.
 */

#include "zh_rf24.h"

/// \cond
#define ZH_RF24_DATA_SEND_SUCCESS BIT0
#define ZH_RF24_DATA_SEND_FAIL BIT1
#define ZH_RF24_MAX_MESSAGE_SIZE 32
/// \endcond

static void s_zh_rf24_processing(void *pvParameter);

static const char *TAG = "zh_rf24";

static EventGroupHandle_t s_zh_rf24_send_cb_status_event_group_handle = {0};
static QueueHandle_t s_zh_rf24_queue_handle = {0};
static TaskHandle_t s_zh_rf24_processing_task_handle = {0};
static zh_rf24_init_config_t s_zh_rf24_init_config = {0};
static spi_device_handle_t s_zh_spi_device_handle = {0};
static bool s_zh_rf24_is_initialized = false;

/// \cond
typedef enum
{
    ZH_RF24_RECV,
    ZH_RF24_SEND,
} __attribute__((packed)) zh_rf24_queue_id_t;

typedef struct
{
    uint8_t pipe;
    uint8_t *data;
    uint8_t data_len;
} __attribute__((packed)) zh_rf24_queue_data_t;

typedef struct
{
    zh_rf24_queue_id_t id;
    zh_rf24_queue_data_t data;
} __attribute__((packed)) zh_rf24_queue_t;

ESP_EVENT_DEFINE_BASE(ZH_RF24);
/// \endcond

esp_err_t zh_rf24_init(zh_rf24_init_config_t *config)
{
    ESP_LOGI(TAG, "RF24 initialization begin.");
    if (config == NULL)
    {
        ESP_LOGE(TAG, "RF24 initialization fail. Invalid argument.");
        return ESP_ERR_INVALID_ARG;
    }
    s_zh_rf24_init_config = *config;
    s_zh_rf24_send_cb_status_event_group_handle = xEventGroupCreate();
    s_zh_rf24_queue_handle = xQueueCreate(s_zh_rf24_init_config.queue_size, sizeof(zh_rf24_queue_t));
    // if (esp_now_init() != ESP_OK || esp_now_register_send_cb(s_zh_espnow_send_cb) != ESP_OK || esp_now_register_recv_cb(s_zh_espnow_recv_cb) != ESP_OK)
    // {
    //     ESP_LOGE(TAG, "ESP-NOW initialization fail. Internal error.");
    //     return ESP_FAIL;
    // }
    // if (xTaskCreatePinnedToCore(&s_zh_espnow_processing, "NULL", s_zh_espnow_init_config.stack_size, NULL, s_zh_espnow_init_config.task_priority, &s_zh_espnow_processing_task_handle, tskNO_AFFINITY) != pdPASS)
    // {
    //     ESP_LOGE(TAG, "ESP-NOW initialization fail. Internal error.");
    //     return ESP_FAIL;
    // }
    s_zh_rf24_is_initialized = true;
    ESP_LOGI(TAG, "RF24 initialization success.");
    return ESP_OK;
}